@@ -9,11 +9,13 @@
 */
 
 #include <Libkleo/Formatting>
+#include <Libkleo/GnuPG>
 #include <Libkleo/KeyCache>
 #include <Libkleo/KeyGroup>
 #include <Libkleo/KeyResolverCore>
 #include <Libkleo/Test>
 
+#include <QFile>
 #include <QObject>
 #include <QProcess>
 #include <QTest>
@@ -24,6 +26,7 @@
 
 using namespace Kleo;
 using namespace GpgME;
+using namespace Qt::Literals::StringLiterals;
 
 namespace QTest
 {
@@ -67,7 +70,18 @@ inline char *toString(const GpgME::Protocol &t)
 template<>
 inline bool qCompare(GpgME::Protocol const &t1, GpgME::Protocol const &t2, const char *actual, const char *expected, const char *file, int line)
 {
-    return compare_helper(t1 == t2, "Compared values are not the same", toString(t1), toString(t2), actual, expected, file, line);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+    auto formatter = Internal::genericToString<GpgME::Protocol>;
+    return compare_helper(t1 == t2, "Compared values are not the same", &t1, &t2, formatter, formatter, actual, expected, file, line);
+#else
+    auto actualVal = [&t1] {
+        return toString(t1);
+    };
+    auto expectedVal = [&t2] {
+        return toString(t2);
+    };
+    return compare_helper(t1 == t2, "Compared values are not the same", actualVal, expectedVal, actual, expected, file, line);
+#endif
 }
 }
 
@@ -90,11 +104,31 @@ class KeyResolverCoreTest : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
+    void initTestCase()
+    {
+        // Check if we need to create GnuPG's socket directory before running the tests to avoid
+        // a race between gpg and gpgsm (see https://dev.gnupg.org/T7332).
+        // On CI /run/user doesn't exist so that GnuPG falls back to using GNUPGHOME as socket directory
+        // which is already created by QTest::qExtractTestData (and running `gpgconf --create-socketdir`
+        // would fail).
+        if (QFile::exists(u"/run/user"_s)) {
+            // The race is fixed in GnuPG 2.5.2, 2.4.6, and 2.2.45
+            mNeedToCreateSocketDir = !(engineIsVersion(2, 5, 2) || //
+                                       (engineIsVersion(2, 4, 6) && !engineIsVersion(2, 5, 0)) || //
+                                       (engineIsVersion(2, 2, 45) && !engineIsVersion(2, 3, 0)));
+        }
+    }
+
     void init()
     {
         mGnupgHome = QTest::qExtractTestData(QStringLiteral("/fixtures/keyresolvercoretest"));
         qputenv("GNUPGHOME", mGnupgHome->path().toLocal8Bit());
 
+        if (mNeedToCreateSocketDir) {
+            int exitCode = QProcess::execute(Kleo::gpgConfPath(), {u"--create-socketdir"_s});
+            QCOMPARE(exitCode, 0);
+        }
+
         // hold a reference to the key cache to avoid rebuilding while the test is running
         mKeyCache = KeyCache::instance();
         // make sure that the key cache has been populated
@@ -1405,6 +1439,7 @@ private:
     }
 
 private:
+    bool mNeedToCreateSocketDir = false;
     QSharedPointer<QTemporaryDir> mGnupgHome;
     std::shared_ptr<const KeyCache> mKeyCache;
 };
