@@ -301,11 +301,11 @@ public:
         QAbstractButton *pgpBtn;
         QAbstractButton *smimeBtn;
         if (mAllowMixed) {
-            pgpBtn = new QCheckBox(i18n("OpenPGP"));
-            smimeBtn = new QCheckBox(i18n("S/MIME"));
+            pgpBtn = new QCheckBox(i18nc("@option:check", "OpenPGP"));
+            smimeBtn = new QCheckBox(i18nc("@option:check", "S/MIME"));
         } else {
-            pgpBtn = new QRadioButton(i18n("OpenPGP"));
-            smimeBtn = new QRadioButton(i18n("S/MIME"));
+            pgpBtn = new QRadioButton(i18nc("@option:radio", "OpenPGP"));
+            smimeBtn = new QRadioButton(i18nc("@option:radio", "S/MIME"));
         }
 #ifndef NDEBUG
         pgpBtn->setObjectName(QLatin1StringView("openpgp button"));
@@ -436,7 +436,7 @@ public:
     void checkAccepted()
     {
         if (mLastError) {
-            KMessageBox::error(q, Formatting::errorAsString(mLastError), i18n("Operation Failed"));
+            KMessageBox::error(q, Formatting::errorAsString(mLastError), i18nc("@title:window", "Operation Failed"));
             mRunningJobs.clear();
             return;
         }
