@@ -7,17 +7,20 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "progressbar.h"
 
+#include <kleo_ui_debug.h>
+
 #include <QTimer>
-#include "kleo_ui_debug.h"
 
 static const int busyTimerTickInterval = 100;
 static const int busyTimerTickIncrement = 5;
 
 Kleo::ProgressBar::ProgressBar(QWidget *parent)
-    : QProgressBar(parent),
-      mRealProgress(-1)
+    : QProgressBar(parent)
+    , mRealProgress(-1)
 {
     mBusyTimer = new QTimer(this);
     connect(mBusyTimer, &QTimer::timeout, this, &ProgressBar::slotBusyTimerTick);
@@ -96,3 +99,4 @@ void Kleo::ProgressBar::fixup(bool newValue)
     }
 }
 
+#include "moc_progressbar.cpp"
