@@ -46,6 +46,8 @@
 using namespace GpgME;
 using namespace Kleo;
 
+using namespace Qt::Literals::StringLiterals;
+
 namespace
 {
 QIcon iconForValidityAndCompliance(UserID::Validity validity, bool isCompliant)
@@ -393,7 +395,7 @@ static QString time_t2string(time_t t)
 
 static QString make_red(const QString &txt)
 {
-    return QLatin1StringView("<font color=\"red\">") + txt.toHtmlEscaped() + QLatin1String("</font>");
+    return QLatin1StringView("<font color=\"red\">") + txt.toHtmlEscaped() + QLatin1StringView("</font>");
 }
 
 }
@@ -409,12 +411,12 @@ static QString toolTipInternal(const GpgME::Key &key, const GpgME::UserID &userI
     QString result;
     if (flags & Formatting::Validity) {
         if (key.protocol() == GpgME::OpenPGP || (key.keyListMode() & Validate)) {
-            if (userID.isRevoked() || key.isRevoked()) {
+            if (key.isDisabled()) {
+                result = i18n("Disabled");
+            } else if (userID.isRevoked() || key.isRevoked()) {
                 result = make_red(i18n("Revoked"));
             } else if (key.isExpired()) {
                 result = make_red(i18n("Expired"));
-            } else if (key.isDisabled()) {
-                result = i18n("Disabled");
             } else if (key.keyListMode() & GpgME::Validate) {
                 if (!userID.isNull()) {
                     if (userID.validity() >= UserID::Validity::Full) {
@@ -495,7 +497,7 @@ static QString toolTipInternal(const GpgME::Key &key, const GpgME::UserID &userI
         result += format_row(i18n("Usage"), format_keyusage(key));
     }
     if (flags & Formatting::KeyID) {
-        result += format_row(i18n("Key ID"), QString::fromLatin1(key.shortKeyID()));
+        result += format_row(i18n("Key ID"), QString::fromLatin1(key.keyID()));
     }
     if (flags & Formatting::Fingerprint) {
         result += format_row(i18n("Fingerprint"), key.primaryFingerprint());
@@ -869,19 +871,19 @@ QString Formatting::ownerTrustShort(Key::OwnerTrust trust)
 
 QString Formatting::validityShort(const Subkey &subkey)
 {
+    if (subkey.isDisabled()) {
+        return i18n("disabled");
+    }
     if (subkey.isRevoked()) {
         return i18n("revoked");
     }
     if (subkey.isExpired()) {
         return i18n("expired");
     }
-    if (subkey.isDisabled()) {
-        return i18n("disabled");
-    }
     if (subkey.isInvalid()) {
         return i18n("invalid");
     }
-    return i18nc("as in good/valid signature", "good");
+    return i18nc("as in 'this subkey is ok'", "OK");
 }
 
 QString Formatting::validityShort(const UserID &uid)
@@ -943,12 +945,12 @@ QString Formatting::validityShort(const UserID::Signature &sig)
         const auto key = KeyCache::instance()->findByKeyIDOrFingerprint(sig.signerKeyID());
         if (key.isNull()) {
             return i18n("no public key");
-        } else if (key.isExpired()) {
-            return i18n("key expired");
-        } else if (key.isRevoked()) {
-            return i18n("key revoked");
         } else if (key.isDisabled()) {
             return i18n("key disabled");
+        } else if (key.isRevoked()) {
+            return i18n("key revoked");
+        } else if (key.isExpired()) {
+            return i18n("key expired");
         }
         /* can't happen */
         return QStringLiteral("unknown");
@@ -1004,7 +1006,7 @@ QString Formatting::formatForComboBox(const GpgME::Key &key)
     if (!mail.isEmpty()) {
         mail = QLatin1Char('<') + mail + QLatin1Char('>');
     }
-    return i18nc("name, email, key id", "%1 %2 (%3)", name, mail, QLatin1StringView(key.shortKeyID())).simplified();
+    return i18nc("name, email, key id", "%1 %2 (%3)", name, mail, QLatin1StringView(key.keyID())).simplified();
 }
 
 QString Formatting::nameAndEmailForSummaryLine(const UserID &id)
@@ -1290,22 +1292,12 @@ QIcon Formatting::validityIcon(const KeyGroup &group)
     return iconForValidityAndCompliance(minimalValidity(group.keys()), allKeysAreCompliant(group.keys()));
 }
 
-bool Formatting::uidsHaveFullValidity(const Key &key)
-{
-    return allUserIDsHaveFullValidity(key);
-}
-
 QString Formatting::complianceMode()
 {
     const auto complianceValue = getCryptoConfigStringValue("gpg", "compliance");
     return complianceValue == QLatin1StringView("gnupg") ? QString() : complianceValue;
 }
 
-bool Formatting::isKeyDeVs(const GpgME::Key &key)
-{
-    return DeVSCompliance::allSubkeysAreCompliant(key);
-}
-
 QString Formatting::complianceStringForKey(const GpgME::Key &key)
 {
     // There will likely be more in the future for other institutions
@@ -1339,14 +1331,14 @@ QString Formatting::complianceStringShort(const GpgME::UserID &id)
     if (keyValidityChecked && id.validity() >= UserID::Full) {
         return i18nc("As in 'this user ID is valid.'", "certified");
     }
-    if (id.parent().isExpired() || isExpired(id)) {
-        return i18n("expired");
+    if (id.parent().isDisabled()) {
+        return i18n("disabled");
     }
     if (id.parent().isRevoked() || id.isRevoked()) {
         return i18n("revoked");
     }
-    if (id.parent().isDisabled()) {
-        return i18n("disabled");
+    if (id.parent().isExpired() || isExpired(id)) {
+        return i18n("expired");
     }
     if (id.parent().isInvalid() || id.isInvalid()) {
         return i18n("invalid");
@@ -1364,14 +1356,14 @@ QString Formatting::complianceStringShort(const GpgME::Key &key)
         return QStringLiteral("★ ") + DeVSCompliance::name(true);
     }
     const bool keyValidityChecked = (key.keyListMode() & GpgME::Validate);
-    if (key.isExpired()) {
-        return i18n("expired");
+    if (key.isDisabled()) {
+        return i18n("disabled");
     }
     if (key.isRevoked()) {
         return i18n("revoked");
     }
-    if (key.isDisabled()) {
-        return i18n("disabled");
+    if (key.isExpired()) {
+        return i18n("expired");
     }
     if (key.isInvalid()) {
         return i18n("invalid");
@@ -1403,8 +1395,14 @@ QString Formatting::prettyID(const char *id)
     if (!id) {
         return QString();
     }
-    QString ret = QString::fromLatin1(id).toUpper().replace(QRegularExpression(QStringLiteral("(....)")), QStringLiteral("\\1 ")).trimmed();
-    // For the standard 10 group fingerprint let us use a double space in the
+    QString ret = QString::fromLatin1(id).toUpper();
+    if (ret.size() == 64) {
+        // looks like a V5 fingerprint; format the first 25 bytes as 10 groups of 5 hex characters
+        ret.truncate(50);
+        return ret.replace(QRegularExpression(QStringLiteral("(.....)")), QStringLiteral("\\1 ")).trimmed();
+    }
+    ret = ret.replace(QRegularExpression(QStringLiteral("(....)")), QStringLiteral("\\1 ")).trimmed();
+    // For the standard 10 group V4 fingerprint let us use a double space in the
     // middle to increase readability
     if (ret.size() == 49) {
         ret.insert(24, QLatin1Char(' '));
@@ -1415,9 +1413,14 @@ QString Formatting::prettyID(const char *id)
 QString Formatting::accessibleHexID(const char *id)
 {
     static const QRegularExpression groupOfFourRegExp{QStringLiteral("(?:(.)(.)(.)(.))")};
+    static const QRegularExpression groupOfFiveRegExp{QStringLiteral("(?:(.)(.)(.)(.)(.))")};
 
     QString ret;
     ret = QString::fromLatin1(id);
+    if (ret.size() == 64) {
+        ret.truncate(50);
+        return ret.replace(groupOfFiveRegExp, QStringLiteral("\\1 \\2 \\3 \\4 \\5, ")).chopped(2);
+    }
     if (!ret.isEmpty() && (ret.size() % 4 == 0)) {
         ret = ret.replace(groupOfFourRegExp, QStringLiteral("\\1 \\2 \\3 \\4, ")).chopped(2);
     }
@@ -1442,7 +1445,7 @@ QString Formatting::origin(int o)
     case Key::OriginOther:
     case Key::OriginUnknown:
     default:
-        return i18n("Unknown");
+        return {};
     }
 }
 
@@ -1544,3 +1547,204 @@ QString Formatting::prettyAlgorithmName(const std::string &algorithm)
     const auto it = displayNames.find(algorithm);
     return (it != displayNames.end()) ? it->second : i18nc("@info", "Unknown algorithm");
 }
+
+static QString formatValidSignatureWithTrustLevel(const GpgME::UserID &id)
+{
+    if (id.isNull()) {
+        return QString();
+    }
+    switch (id.validity()) {
+    case GpgME::UserID::Marginal:
+        return i18n("The signature is valid but the trust in the certificate's validity is only marginal.");
+    case GpgME::UserID::Full:
+        return i18n("The signature is valid and the certificate's validity is fully trusted.");
+    case GpgME::UserID::Ultimate:
+        return i18n("The signature is valid and the certificate's validity is ultimately trusted.");
+    case GpgME::UserID::Never:
+        return i18n("The signature is valid but the certificate's validity is <em>not trusted</em>.");
+    case GpgME::UserID::Unknown:
+        return i18n("The signature is valid but the certificate's validity is unknown.");
+    case GpgME::UserID::Undefined:
+    default:
+        return i18n("The signature is valid but the certificate's validity is undefined.");
+    }
+}
+
+static QString renderKeyLink(const QString &fpr, const QString &text)
+{
+    return QStringLiteral("<a href=\"key:%1\">%2</a>").arg(fpr, text.toHtmlEscaped());
+}
+
+static QString renderKey(const GpgME::Key &key)
+{
+    if (key.isNull()) {
+        return i18n("Unknown certificate");
+    }
+
+    return renderKeyLink(QLatin1StringView(key.primaryFingerprint()),
+                         i18nc("User ID (Key ID)", "%1 (%2)", Formatting::prettyNameAndEMail(key), Formatting::prettyID(key.subkey(0).keyID())));
+}
+
+static QString formatSigningInformation(const GpgME::Signature &sig, const GpgME::Key &key)
+{
+    if (sig.isNull()) {
+        return QString();
+    }
+    QString text;
+    const QDateTime dt = sig.creationTime() != 0 ? QDateTime::fromSecsSinceEpoch(quint32(sig.creationTime())) : QDateTime();
+
+    if (key.isNull()) {
+        const QString id = u"<br/>"_s + Formatting::prettyID(sig.fingerprint());
+        if (dt.isValid()) {
+            return i18nc("1 is a date", "Signature created on %1 using an unknown certificate with fingerprint %2", QLocale().toString(dt), id);
+        }
+        return i18n("Signature created using an unknown certificate with fingerprint %1", id);
+    }
+
+    if (dt.isValid()) {
+        text += i18nc("1 is a date", "Signature created on %1 with certificate: %2", QLocale().toString(dt), renderKey(key));
+    } else {
+        text += i18n("Signature created with certificate: %1", renderKey(key));
+    }
+
+    if (Kleo::DeVSCompliance::isCompliant() && ((sig.summary() & GpgME::Signature::Valid) || (sig.summary() & GpgME::Signature::Green))) {
+        text += (QStringLiteral("<br/>")
+                 + (sig.isDeVs() ? i18nc("%1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
+                                         "The signature is %1",
+                                         Kleo::DeVSCompliance::name(true))
+                                 : i18nc("%1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
+                                         "The signature <b>is not</b> %1.",
+                                         Kleo::DeVSCompliance::name(true))));
+    }
+
+    return text;
+}
+
+static QString signatureSummaryToString(GpgME::Signature::Summary summary)
+{
+    if (summary & GpgME::Signature::None) {
+        return i18n("Error: Signature not verified");
+    } else if ((summary & GpgME::Signature::Valid) || (summary & GpgME::Signature::Green)) {
+        return i18n("Good signature");
+    } else if (summary & GpgME::Signature::KeyRevoked) {
+        return i18n("Signing certificate was revoked");
+    } else if (summary & GpgME::Signature::KeyExpired) {
+        return i18n("Signing certificate is expired");
+    } else if (summary & GpgME::Signature::KeyMissing) {
+        return i18n("Certificate is not available");
+    } else if (summary & GpgME::Signature::SigExpired) {
+        return i18n("Signature expired");
+    } else if (summary & GpgME::Signature::CrlMissing) {
+        return i18n("CRL missing");
+    } else if (summary & GpgME::Signature::CrlTooOld) {
+        return i18n("CRL too old");
+    } else if (summary & GpgME::Signature::BadPolicy) {
+        return i18n("Bad policy");
+    } else if (summary & GpgME::Signature::SysError) {
+        return i18n("System error"); // ### retrieve system error details?
+    } else if (summary & GpgME::Signature::Red) {
+        return i18n("Bad signature");
+    }
+    return QString();
+}
+
+static QLatin1StringView stripAngleBrackets(const QLatin1StringView &str)
+{
+    if (str.isEmpty()) {
+        return str;
+    }
+    if (str[0] == '<' && str[str.size() - 1] == '>') {
+        return str.mid(1, str.size() - 2);
+    }
+    return str;
+}
+
+QString Formatting::email(const GpgME::UserID &uid)
+{
+    if (uid.parent().protocol() == GpgME::OpenPGP) {
+        const QLatin1StringView email(uid.email());
+        if (!email.isEmpty()) {
+            return stripAngleBrackets(email).toString();
+        }
+        return {};
+    }
+
+    Q_ASSERT(uid.parent().protocol() == GpgME::CMS);
+
+    const QLatin1StringView id(uid.id());
+    if (!id.isEmpty()) {
+        if (id[0] == '<') {
+            return stripAngleBrackets(id).toString();
+        }
+        return Kleo::DN(id)[QStringLiteral("EMAIL")].trimmed();
+    }
+    return {};
+}
+
+static GpgME::UserID findUserIDByMailbox(const GpgME::Key &key, const QString &email)
+{
+    const auto userIDs{key.userIDs()};
+    for (const GpgME::UserID &id : userIDs) {
+        if (Formatting::email(id).compare(email, Qt::CaseInsensitive)) {
+            return id;
+        }
+    }
+    return {};
+}
+
+QString Kleo::Formatting::prettySignature(const GpgME::Signature &sig, const QString &sender)
+{
+    if (sig.isNull()) {
+        return QString();
+    }
+
+    const GpgME::Key key = Kleo::KeyCache::instance()->findSigner(sig);
+
+    const QString text = formatSigningInformation(sig, key) + QLatin1StringView("<br/>");
+
+    // Green
+    if (sig.summary() & GpgME::Signature::Valid) {
+        GpgME::UserID id = findUserIDByMailbox(key, sender);
+        if (id.isNull()) {
+            for (int i = 0, count = key.userIDs().size(); i < count; i++) {
+                id = key.userID(i);
+                if (!id.isNull()) {
+                    break;
+                }
+            }
+        }
+
+        return text + formatValidSignatureWithTrustLevel(!id.isNull() ? id : key.userID(0));
+    }
+
+    // Red
+    if ((sig.summary() & GpgME::Signature::Red)) {
+        const QString ret = text + i18n("The signature is invalid: %1", signatureSummaryToString(sig.summary()));
+        if (sig.summary() & GpgME::Signature::SysError) {
+            return ret + QStringLiteral(" (%1)").arg(Kleo::Formatting::errorAsString(sig.status()));
+        }
+        return ret;
+    }
+
+    // Key missing
+    if ((sig.summary() & GpgME::Signature::KeyMissing)) {
+        return text + i18n("You can search the certificate on a keyserver or import it from a file.");
+    }
+
+    // Yellow
+    if ((sig.validity() & GpgME::Signature::Validity::Undefined) //
+        || (sig.validity() & GpgME::Signature::Validity::Unknown) //
+        || (sig.summary() == GpgME::Signature::Summary::None)) {
+        return text
+            + (key.protocol() == GpgME::OpenPGP
+                   ? i18n("The used key is not certified by you or any trusted person.")
+                   : i18n("The used certificate is not certified by a trustworthy Certificate Authority or the Certificate Authority is unknown."));
+    }
+
+    // Catch all fall through
+    const QString ret = text + i18n("The signature is invalid: %1", signatureSummaryToString(sig.summary()));
+    if (sig.summary() & GpgME::Signature::SysError) {
+        return ret + QStringLiteral(" (%1)").arg(Kleo::Formatting::errorAsString(sig.status()));
+    }
+    return ret;
+}
