@@ -7,14 +7,16 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "hex.h"
 
-#include "kleo/kleoexception.h"
+#include <libkleo/kleoexception.h>
 
 #include <KLocalizedString>
 
-#include <QString>
 #include <QByteArray>
+#include <QString>
 
 using namespace Kleo;
 
@@ -30,9 +32,7 @@ static unsigned char unhex(unsigned char ch)
         return ch - 'a' + 10;
     }
     const char cch = ch;
-    throw Kleo::Exception(gpg_error(GPG_ERR_ASS_SYNTAX),
-                    i18n("Invalid hex char '%1' in input stream.",
-                         QString::fromLatin1(&cch, 1)));
+    throw Kleo::Exception(gpg_error(GPG_ERR_ASS_SYNTAX), i18n("Invalid hex char '%1' in input stream.", QString::fromLatin1(&cch, 1)));
 }
 
 std::string Kleo::hexdecode(const std::string &in)
@@ -44,20 +44,18 @@ std::string Kleo::hexdecode(const std::string &in)
             ++it;
             unsigned char ch = '\0';
             if (it == end) {
-                throw Exception(gpg_error(GPG_ERR_ASS_SYNTAX),
-                                i18n("Premature end of hex-encoded char in input stream"));
+                throw Exception(gpg_error(GPG_ERR_ASS_SYNTAX), i18n("Premature end of hex-encoded char in input stream"));
             }
             ch |= unhex(*it) << 4;
             ++it;
             if (it == end) {
-                throw Exception(gpg_error(GPG_ERR_ASS_SYNTAX),
-                                i18n("Premature end of hex-encoded char in input stream"));
+                throw Exception(gpg_error(GPG_ERR_ASS_SYNTAX), i18n("Premature end of hex-encoded char in input stream"));
             }
             ch |= unhex(*it);
             result.push_back(ch);
         } else if (*it == '+') {
             result += ' ';
-        } else  {
+        } else {
             result.push_back(*it);
         }
     }
@@ -90,8 +88,8 @@ std::string Kleo::hexencode(const std::string &in)
         case '+':
         case '=':
             result += '%';
-            result += hex[(ch & 0xF0) >> 4 ];
-            result += hex[(ch & 0x0F)      ];
+            result += hex[(ch & 0xF0) >> 4];
+            result += hex[(ch & 0x0F)];
             break;
         }
     }
