@@ -20,9 +20,6 @@
 
 #include <QDate>
 
-// needed for GPGME_VERSION_NUMBER
-#include <gpgme.h>
-
 #include <iterator>
 
 using namespace Kleo;
@@ -42,7 +39,7 @@ auto _getMissingSignerKeyIds(const std::vector<GpgME::UserID::Signature> &signat
 {
     return std::accumulate(std::begin(signatures), std::end(signatures), std::set<QString>{}, [](auto &keyIds, const auto &signature) {
         if (!havePublicKeyForSignature(signature)) {
-            keyIds.insert(QLatin1String{signature.signerKeyID()});
+            keyIds.insert(QLatin1StringView{signature.signerKeyID()});
         }
         return keyIds;
     });
@@ -141,6 +138,20 @@ bool Kleo::isRevokedOrExpired(const GpgME::UserID &userId)
     return !sig.isNull() && (sig.isRevokation() || sig.isExpired());
 }
 
+bool Kleo::isExpired(const UserID &userID)
+{
+    if (userID.parent().isExpired()) {
+        return true;
+    }
+    const auto sigs = userID.signatures();
+    std::vector<GpgME::UserID::Signature> selfSigs;
+    std::copy_if(std::begin(sigs), std::end(sigs), std::back_inserter(selfSigs), &Kleo::isSelfSignature);
+    std::sort(std::begin(selfSigs), std::end(selfSigs));
+    // check the most recent signature
+    const auto sig = !selfSigs.empty() ? selfSigs.back() : GpgME::UserID::Signature{};
+    return !sig.isNull() && sig.isExpired();
+}
+
 bool Kleo::canCreateCertifications(const GpgME::Key &key)
 {
     return Kleo::keyHasCertify(key) && canBeUsedForSecretKeyOperations(key);
@@ -153,20 +164,6 @@ bool Kleo::canBeCertified(const GpgME::Key &key)
         && hasValidUserID(key);
 }
 
-namespace
-{
-static inline bool subkeyHasSecret(const GpgME::Subkey &subkey)
-{
-#if GPGME_VERSION_NUMBER >= 0x011102 // 1.17.2
-    // we need to check the primary subkey because Key::hasSecret() is also true if just the secret key stub of an offline key is available
-    return subkey.isSecret();
-#else
-    // older versions of GpgME did not always set the secret flag for card keys
-    return subkey.isSecret() || subkey.isCardKey();
-#endif
-}
-}
-
 bool Kleo::canBeUsedForEncryption(const GpgME::Key &key)
 {
     return !key.isBad() && Kleo::any_of(key.subkeys(), [](const auto &subkey) {
@@ -177,13 +174,14 @@ bool Kleo::canBeUsedForEncryption(const GpgME::Key &key)
 bool Kleo::canBeUsedForSigning(const GpgME::Key &key)
 {
     return !key.isBad() && Kleo::any_of(key.subkeys(), [](const auto &subkey) {
-        return subkey.canSign() && !subkey.isBad() && subkeyHasSecret(subkey);
+        return subkey.canSign() && !subkey.isBad() && subkey.isSecret();
     });
 }
 
 bool Kleo::canBeUsedForSecretKeyOperations(const GpgME::Key &key)
 {
-    return subkeyHasSecret(key.subkey(0));
+    // we need to check the primary subkey because Key::hasSecret() is also true if just the secret key stub of an offline key is available
+    return key.subkey(0).isSecret();
 }
 
 bool Kleo::canRevokeUserID(const GpgME::UserID &userId)
