@@ -284,7 +284,7 @@ int main(int argc, char **argv)
             Q_ASSERT(entry);
             Q_ASSERT(entry->argType() == QGpgME::CryptoConfigEntry::ArgType_Path);
             cout << "Log-file now: " << entry->stringValue().toLocal8Bit().constData() << endl;
-            Q_ASSERT(entry->stringValue() == QStringLiteral("/tmp/test:%e5ä")); // (or even with %e5 decoded)
+            Q_ASSERT(entry->stringValue() == QLatin1StringView("/tmp/test:%e5ä")); // (or even with %e5 decoded)
 
             // Reset old value
 #if 0
@@ -325,11 +325,11 @@ int main(int argc, char **argv)
             lst << QUrl(QStringLiteral("ldap://a:389?b"));
             Q_ASSERT(lst[0].query() == QLatin1Char('b'));
             lst << QUrl(QStringLiteral("ldap://foo:389?a:b c"));
-            Q_ASSERT(lst[1].query() == QStringLiteral("a:b c")); // see, the space got _not_escaped
+            Q_ASSERT(lst[1].query() == QLatin1StringView("a:b c")); // see, the space got _not_escaped
             lst << QUrl(QStringLiteral("ldap://server:389?a=b,c=DE"));
-            Q_ASSERT(lst[2].query() == QStringLiteral("a=b,c=DE")); // the query contains a literal ','
+            Q_ASSERT(lst[2].query() == QLatin1StringView("a=b,c=DE")); // the query contains a literal ','
             lst << QUrl(QStringLiteral("ldap://foo:389?a#ldaps"));
-            Q_ASSERT(lst[3].fragment() == QStringLiteral("ldaps"));
+            Q_ASSERT(lst[3].fragment() == QLatin1StringView("ldaps"));
             // cout << " trying to set: " << lst.toStringList().join(", ").local8Bit() << endl;
             entry->setURLValueList(lst);
             Q_ASSERT(entry->isDirty());
